"""Metadata package for libtmux."""

from __future__ import annotations

__title__ = "libtmux"
__package_name__ = "libtmux"
__version__ = "0.62.0"
__description__ = "Typed scripting library / ORM / API wrapper for tmux"
__email__ = "tony@git-pull.com"
__author__ = "Tony Narlock"
__github__ = "https://github.com/tmux-python/libtmux"
__docs__ = "https://libtmux.git-pull.com"
__tracker__ = "https://github.com/tmux-python/libtmux/issues"
__pypi__ = "https://pypi.org/project/libtmux/"
__license__ = "MIT"
__copyright__ = "Copyright 2016- Tony Narlock"
